// MediaViewer.cpp: CMediaViewer NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MediaViewer.h"
#include <Dvdmedia.h>
#ifdef VMR9_SUPPORTED
#include <d3d9.h>
#include <Vmr9.h>
#endif
#include "../Grabber.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CMediaViewer::CMediaViewer(IEventHandler *pEventHandler)
	: CMediaDecoder(pEventHandler, 1UL, 0UL)
	, m_bInit(false)
	, m_pMediaControl(NULL)

	, m_pFilterGraph(NULL)

	, m_pSrcFilter(NULL)
	, m_pBonSrcFilterClass(NULL)

	, m_pMpeg2SeqFilter(NULL)
	, m_pMpeg2SeqClass(NULL)

	, m_pAacDecFilter(NULL)
	, m_pAacDecClass(NULL)

	, m_pPcmSelFilter(NULL)
	, m_pPcmSelClass(NULL)

	, m_pMp2DemuxFilter(NULL)
	, m_pMpeg2DecFilter(NULL)

	, m_pVideoRenderer(NULL)

	, m_strMpeg2DecorderName()

	, m_pMp2DemuxInterface(NULL)
	, m_pMp2DemuxRefClock(NULL)

	, m_pMp2DemuxVideoMap(NULL)
	, m_pMp2DemuxAudioMap(NULL)

	, m_pVideoWindow(NULL)
	, m_pBasicVideo(NULL)
	, m_pBasicAudio(NULL)
	
	, m_wVideoEsPID(0xFFFFU)
	, m_wAudioEsPID(0xFFFFU)
	
	, m_wVideoWindowX(0)
	, m_wVideoWindowY(0)
	, m_VideoInfo()
	, m_hOwnerWnd(NULL)
	, m_dwRegister(0)

	, m_ForceAspectX(0)
	, m_ForceAspectY(0)
	, m_PanAndScan(0)
#ifdef VMR9_SUPPORTED
	, m_bVMR9(false)
	, m_pImageMixer(NULL)
#endif
	, m_bGrabber(false)
	, m_pGrabber(NULL)
	, m_pTracer(NULL)
{
	// COMCu
	::CoInitialize(NULL);
}

CMediaViewer::~CMediaViewer()
{
	CloseViewer();

	// COMCuJ
	::CoUninitialize();
}

void CMediaViewer::Reset(void)
{
	Stop();

	CMediaDecoder::Reset();

	SetVideoPID(0xFFFFU);
	SetAudioPID(0xFFFFU);

	Play();
}

const bool CMediaViewer::InputMedia(CMediaData *pMediaData, const DWORD dwInputIndex)
{
	if(dwInputIndex > GetInputNum())return false;

	CTsPacket *pTsPacket = dynamic_cast<CTsPacket *>(pMediaData);

	// ̓fBAf[^͌݊Ȃ
	if(!pTsPacket)return false;
	
	// tB^Otɓ
	if(m_pBonSrcFilterClass){
		return m_pBonSrcFilterClass->InputMedia(pTsPacket);
		}

	return false;
}

const bool CMediaViewer::OpenViewer(HWND hOwnerHwnd,HWND hMessageDrainHwnd,
									LPCWSTR pszMpeg2Decoder)
{
	if (m_bInit)
		return false;

	HRESULT hReturn=S_OK;
	ULONG refCount;

	CMediaType MediaTypeVideo;
	CMediaType MediaTypeAudio;

	IPin *pOutput=NULL;
	IPin *pOutputVideo=NULL;
	IPin *pOutputAudio=NULL;

	try {
		CDirectShowUtil cDSUtil;

		// tB^Ot}l[W\z
		if (::CoCreateInstance(CLSID_FilterGraph,NULL,CLSCTX_INPROC_SERVER,
				IID_IGraphBuilder,reinterpret_cast<LPVOID*>(&m_pFilterGraph)) != S_OK)
			throw 1UL;
#ifdef DEBUG
		AddToRot(m_pFilterGraph, &m_dwRegister);
#endif

		// IMediaControlC^tF[X̃NG[
		if (m_pFilterGraph->QueryInterface(IID_IMediaControl, reinterpret_cast<LPVOID*>(&m_pMediaControl)) != S_OK)
			throw 1UL;

		Trace(TEXT("\[XtB^̐ڑ..."));

		/* CBonSrcFilter */
		{
			// CX^X쐬
			m_pSrcFilter = CBonSrcFilter::CreateInstance(NULL, &hReturn, &m_pBonSrcFilterClass);
			if((!m_pSrcFilter) || (hReturn != S_OK))throw 1UL;
			//refCount = CDirectShowUtil::GetRefCount(m_pSrcFilter);
			// tB^Otɒǉ
			if(m_pFilterGraph->AddFilter(m_pSrcFilter, L"BonSrcFilter") != S_OK)
				throw 1UL;
			//refCount = CDirectShowUtil::GetRefCount(m_pSrcFilter);
			// o̓s擾
			pOutput = CDirectShowUtil::GetFilterPin(m_pSrcFilter,PINDIR_OUTPUT);
			if(!pOutput) 1UL;
			//refCount = CDirectShowUtil::GetRefCount(m_pSrcFilter);
		}

		Trace(TEXT("MPEG-2 DemuxertB^̐ڑ..."));

		/* MPEG-2 Demultiplexer */
		{
			// MPEG-2 DemultiplexerCX^X쐬āAڑs
			if(!CDirectShowUtil::AppendFilter_and_Connect(CLSID_MPEG2Demultiplexer,
					L"Mpeg2Demuxer", m_pFilterGraph, &m_pMp2DemuxFilter , &pOutput))
				throw 1UL;
			// ̎_pOutput==NULL̂͂Ô
			SAFE_RELEASE(pOutput);
			// IMpeg2DemultiplexerC^tF[X̃NG[
			if(m_pMp2DemuxFilter->QueryInterface(IID_IMpeg2Demultiplexer, (void**)&m_pMp2DemuxInterface) != S_OK)
				throw 1UL;
			// IReferenceClockC^tF[X̃NG[
			if(m_pMp2DemuxFilter->QueryInterface(IID_IReferenceClock, (void**)&m_pMp2DemuxRefClock) != S_OK)
				throw 1UL;
			// t@XNbNI
			if(m_pMp2DemuxFilter->SetSyncSource(m_pMp2DemuxRefClock) != S_OK)
				throw 1UL;
			// ffBAtH[}bgݒ
			MediaTypeVideo.InitMediaType();
			MediaTypeVideo.SetType(&MEDIATYPE_Video);
			MediaTypeVideo.SetSubtype(&MEDIASUBTYPE_MPEG2_VIDEO);
			MediaTypeVideo.SetVariableSize();
			MediaTypeVideo.SetTemporalCompression(TRUE);
			MediaTypeVideo.SetSampleSize(0);
			MediaTypeVideo.SetFormatType(&FORMAT_MPEG2Video);
			 // tH[}bg\̊m
			MPEG2VIDEOINFO *pVideoInfo = (MPEG2VIDEOINFO *)MediaTypeVideo.AllocFormatBuffer(sizeof(MPEG2VIDEOINFO));
			if(!pVideoInfo) throw 1UL;
			::ZeroMemory(pVideoInfo, sizeof(MPEG2VIDEOINFO));
			// rfIwb_ݒ
			VIDEOINFOHEADER2 &VideoHeader = pVideoInfo->hdr;
			::SetRect(&VideoHeader.rcSource, 0, 0, 720, 480);
			VideoHeader.bmiHeader.biHeight = 480;
			VideoHeader.bmiHeader.biWidth = 720;
			// fo̓s쐬
			if (m_pMp2DemuxInterface->CreateOutputPin(&MediaTypeVideo,L"Video",&pOutputVideo) != S_OK)
				throw 1UL;
			// fBAtH[}bgݒ	
			MediaTypeAudio.InitMediaType();
			MediaTypeAudio.SetType(&MEDIATYPE_Audio);
			MediaTypeAudio.SetSubtype(&MEDIASUBTYPE_NULL);
			MediaTypeAudio.SetVariableSize();
			MediaTypeAudio.SetTemporalCompression(TRUE);
			MediaTypeAudio.SetSampleSize(0);
			MediaTypeAudio.SetFormatType(&FORMAT_None);
			// o̓s쐬
			if (m_pMp2DemuxInterface->CreateOutputPin(&MediaTypeAudio,L"Audio",&pOutputAudio) != S_OK)
				throw 1UL;
			// fo̓sIMPEG2PIDMapC^tF[X̃NG[
			if (pOutputVideo->QueryInterface(IID_IMPEG2PIDMap,(void**)&m_pMp2DemuxVideoMap) != S_OK)
				throw 1UL;
			// o̓sIMPEG2PIDMapC^tF[X̃NG
			if (pOutputAudio->QueryInterface(IID_IMPEG2PIDMap,(void**)&m_pMp2DemuxAudioMap) != S_OK)
				throw 1UL;
		}

		Trace(TEXT("MPEG-2V[PXtB^̐ڑ..."));

		/* CMpeg2SequenceFilter */
		{
			// CX^X쐬
			m_pMpeg2SeqFilter = CMpeg2SequenceFilter::CreateInstance(NULL, &hReturn,&m_pMpeg2SeqClass);
			if((!m_pMpeg2SeqFilter) || (hReturn != S_OK))
				throw 1UL;
			m_pMpeg2SeqClass->SetRecvCallback(OnMpeg2VideoInfo,this);
			// tB^̒ǉƐڑ
			if (!CDirectShowUtil::AppendFilter_and_Connect(m_pMpeg2SeqFilter,
					L"Mpeg2SequenceFilter", m_pFilterGraph, &pOutputVideo))
				throw 1UL;
		}

		Trace(TEXT("AACfR[_̐ڑ..."));

		/* CAacDecFilter */
		{
			// CAacDecFilterCX^X쐬
			m_pAacDecFilter=CAacDecFilter::CreateInstance(NULL,&hReturn,&m_pAacDecClass);
			if (!m_pAacDecFilter || hReturn!=S_OK)
				throw 1UL;
			// tB^̒ǉƐڑ
			//refCount = CDirectShowUtil::GetRefCount(m_pAacDecFilter);
			if (!CDirectShowUtil::AppendFilter_and_Connect(m_pAacDecFilter,
					L"AacDecFilter",m_pFilterGraph,&pOutputAudio))
				throw 1UL;
		}

		Trace(TEXT("PCMZNgtB^̐ڑ..."));

		/* CPcmSelectFilter */
		{
			// CX^X쐬
			m_pPcmSelFilter=CPcmSelectFilter::CreateInstance(NULL,&hReturn,&m_pPcmSelClass);
			if (!m_pPcmSelFilter || hReturn!=S_OK)
				throw 1UL;
			// tB^̒ǉƐڑ
			if (!CDirectShowUtil::AppendFilter_and_Connect(m_pPcmSelFilter,
					L"PcmSelFilter",m_pFilterGraph,&pOutputAudio))
				throw 1UL;
		}

		Trace(TEXT("MPEG-2fR[_̐ڑ..."));

		/* Mpeg2fR[_[ */
		{
			// 
			if(!cDSUtil.FindFilter(&MEDIATYPE_Video,&MEDIASUBTYPE_MPEG2_VIDEO)) throw 1UL;

			WCHAR szMpeg2Vid[128];
			CLSID idMpeg2Vid;
			int i;

			bool bConnectSuccess=false;
			for (i=0;!bConnectSuccess && i<cDSUtil.GetFilterCount();i++){
				if (cDSUtil.GetFilterInfo(i,&idMpeg2Vid,szMpeg2Vid,128)) {
					if (pszMpeg2Decoder!=NULL && pszMpeg2Decoder[0]!='\0'
							&& lstrcmpi(szMpeg2Vid,pszMpeg2Decoder)!=0)
						continue;
					/*
					if (::CoCreateInstance(idMpeg2Vid,NULL,CLSCTX_INPROC_SERVER,IID_IBaseFilter, reinterpret_cast<LPVOID*>(&m_pMpeg2DecFilter)) == S_OK){
						// Mpeg2fR[_[tB^Otɒǉ
						
						if(m_pFilterGraph->AddFilter(static_cast<IBaseFilter *>(m_pMpeg2DecFilter), szMpeg2Vid) == S_OK){
							// Mpeg2fR[_[s
							pMpeg2InPin = cDSUtil.GetFilterPin(m_pMpeg2DecFilter,PINDIR_INPUT,NULL);
							pMpeg2OutPin = cDSUtil.GetFilterPin(m_pMpeg2DecFilter,PINDIR_OUTPUT,NULL);

							// sڑ(CMpeg2SequenceFilter  [] Mpeg2Decorder)
							if(m_pFilterGraph->ConnectDirect(pOutputVideo, pMpeg2InPin, NULL) == S_OK){
								// sڑ
								SAFE_RELEASE(pOutputVideo);
								pOutputVideo=pMpeg2OutPin;
								bConnectSuccess = true;
							} else {
								// sڑŝ߁AtB^Ot菜
								m_pFilterGraph->RemoveFilter(m_pMpeg2DecFilter);
								SAFE_RELEASE(pMpeg2InPin);
								SAFE_RELEASE(pMpeg2OutPin);
								SAFE_RELEASE(m_pMpeg2DecFilter);
							}
						} else {
							// tB^Otɒǉs
							SAFE_RELEASE(m_pMpeg2DecFilter);
						}
					} else {
						// fR[_[̃CX^X쐬s
					}
					*/
					if (cDSUtil.AppendFilter_and_Connect(idMpeg2Vid,szMpeg2Vid,
							m_pFilterGraph,&m_pMpeg2DecFilter,&pOutputVideo)) {
						bConnectSuccess=true;
						break;
					}
				} else {
					// tB^擾s
				}
			}
			// ǂꂩ̃tB^Őڑł
			if (bConnectSuccess) {
				m_strMpeg2DecorderName = szMpeg2Vid;
			} else {
				m_strMpeg2DecorderName = L"";
				throw 1UL;
			}
		}

#if 1	// OoeXg܂܂Ȃ̂ŕۗ
		if (m_bGrabber) {
			m_pGrabber=new CGrabber();
			IBaseFilter *pGrabberFilter;

			Trace(TEXT("OotB^ڑ..."));

			m_pGrabber->Init();
			pGrabberFilter=m_pGrabber->GetGrabberFilter();
			if (!cDSUtil.AppendFilter_and_Connect(pGrabberFilter,L"Grabber",
											m_pFilterGraph,&pOutputVideo)) {
				delete m_pGrabber;
				m_pGrabber=NULL;
			}
		}
#endif

#ifdef VMR9_SUPPORTED
		if (m_bVMR9) {
			Trace(TEXT("VMR-9̏..."));
			if (SUCCEEDED(::CoCreateInstance(CLSID_VideoMixingRenderer9,NULL,
					CLSCTX_INPROC_SERVER,IID_IBaseFilter,
					reinterpret_cast<LPVOID*>(&m_pVideoRenderer)))) {
				if (SUCCEEDED(m_pFilterGraph->AddFilter(m_pVideoRenderer,L"VMR9"))) {
					IVMRFilterConfig *pFilterConfig;
					IVMRWindowlessControl9 *pWindowlessControl;
					IVMRMixerControl9 *pMixerControl;
					RECT rc;

					m_pVideoRenderer->QueryInterface(IID_IVMRFilterConfig9,
									reinterpret_cast<LPVOID*>(&pFilterConfig));
					pFilterConfig->SetRenderingMode(VMRMode_Windowless);
					pFilterConfig->Release();
					m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl));
					pWindowlessControl->SetVideoClippingWindow(hOwnerHwnd);
					pWindowlessControl->SetBorderColor(RGB(0,0,0));
					pWindowlessControl->SetAspectRatioMode(VMR9ARMode_None);
					::GetClientRect(hOwnerHwnd,&rc);
					pWindowlessControl->SetVideoPosition(NULL,&rc);
					pWindowlessControl->Release();
					if (SUCCEEDED(m_pVideoRenderer->QueryInterface(IID_IVMRMixerControl9,
							reinterpret_cast<LPVOID*>(&pMixerControl)))) {
						DWORD MixingPref;
						pMixerControl->GetMixingPrefs(&MixingPref);
						MixingPref=(MixingPref&~MixerPref9_DecimateMask)|MixerPref9_NonSquareMixing;
						pMixerControl->SetMixingPrefs(MixingPref);
						pMixerControl->Release();
					}
				} else {
					SAFE_RELEASE(m_pVideoRenderer);
				}
			}
		}
#endif	// VMR9_SUPPORTED

		Trace(TEXT("f_̍\z..."));

#ifdef VMR9_SUPPORTED
		if (m_pVideoRenderer) {
			IFilterGraph2 *pFilterGraph2;

			if (FAILED(m_pFilterGraph->QueryInterface(IID_IFilterGraph2,
								reinterpret_cast<LPVOID*>(&pFilterGraph2)))) {
				throw 1UL;
			}
			if (FAILED(pFilterGraph2->RenderEx(pOutputVideo,
								AM_RENDEREX_RENDERTOEXISTINGRENDERERS,NULL))) {
				pFilterGraph2->Release();
				throw 1UL;
			}
			pFilterGraph2->Release();
		} else
#endif

		// f_[\z
		if(m_pFilterGraph->Render(pOutputVideo) != S_OK)throw 1UL;

		Trace(TEXT("_̍\z..."));

		// _[\z
		if(m_pFilterGraph->Render(pOutputAudio) != S_OK)throw 1UL;		

#ifdef VMR9_SUPPORTED
		if (m_pVideoRenderer==NULL) {
#endif
		// IVideoWindowC^tF[X̃NG[
		if(m_pFilterGraph->QueryInterface(IID_IVideoWindow, reinterpret_cast<LPVOID *>(&m_pVideoWindow)) != S_OK)throw 1UL;		

		// IBasicVideo2C^[tF[X̃NG[
		if(m_pFilterGraph->QueryInterface(IID_IBasicVideo2, reinterpret_cast<LPVOID *>(&m_pBasicVideo)) != S_OK)throw 1UL;
#ifdef VMR9_SUPPORTED
		}
#endif

		// IBasicAudioC^[tF[X̃NG[
		if(m_pFilterGraph->QueryInterface(IID_IBasicAudio, reinterpret_cast<LPVOID *>(&m_pBasicAudio)) != S_OK)throw 1UL;

		// I[iEBhEݒ
		m_hOwnerWnd = hOwnerHwnd;
		RECT Rect;
		::GetClientRect(hOwnerHwnd, &Rect);
		if (m_pVideoWindow && m_pVideoRenderer==NULL) {
			m_pVideoWindow->put_Owner((OAHWND)hOwnerHwnd);
			m_pVideoWindow->put_MessageDrain((OAHWND)hMessageDrainHwnd);
			m_pVideoWindow->put_WindowStyle(WS_CHILD | WS_CLIPSIBLINGS);
			m_pVideoWindow->put_BackgroundPalette(OATRUE);
			m_pVideoWindow->put_BorderColor(RGB(0,0,0));
			m_pVideoWindow->put_Caption(L"");
			m_pVideoWindow->SetWindowPosition(0, 0, Rect.right, Rect.bottom);
			m_pVideoWindow->SetWindowForeground(OATRUE);
			m_pVideoWindow->put_Visible(OATRUE);
		}
		SetViewSize(Rect.right, Rect.bottom);
		m_bInit=true;
	} catch (DWORD dwErrorCode) {
		CloseViewer();
		TRACE(TEXT("tB^Ot\zs[dwErrorCode = %lu]\n"), dwErrorCode);
		return false;
	}

	SAFE_RELEASE(pOutputVideo);
	SAFE_RELEASE(pOutputAudio);

	TRACE(TEXT("tB^Ot\z\n"));
	return true;
}

void CMediaViewer::CloseViewer(void)
{
	if (!m_bInit)
		return;

	Stop();

	// COMCX^XJ

	if(m_pVideoWindow){
		m_pVideoWindow->put_Visible(OAFALSE);
		m_pVideoWindow->put_Owner(NULL);
	}

	SAFE_RELEASE(m_pBasicAudio);
	SAFE_RELEASE(m_pBasicVideo);
	SAFE_RELEASE(m_pVideoWindow);

	SAFE_RELEASE(m_pVideoRenderer);

#ifdef VMR9_SUPPORTED
	if (m_pImageMixer!=NULL) {
		delete m_pImageMixer;
		m_pImageMixer=NULL;
	}
#endif

#if 1
	if (m_pGrabber!=NULL) {
		m_pFilterGraph->RemoveFilter(m_pGrabber->GetGrabberFilter());
		delete m_pGrabber;
		m_pGrabber=NULL;
	}
#endif

	SAFE_RELEASE(m_pMpeg2DecFilter);

	SAFE_RELEASE(m_pPcmSelFilter);
	m_pPcmSelClass=NULL;
	SAFE_RELEASE(m_pAacDecFilter);
	m_pAacDecClass=NULL;

	SAFE_RELEASE(m_pMpeg2SeqFilter);
	m_pMpeg2SeqClass=NULL;

	SAFE_RELEASE(m_pMp2DemuxAudioMap);
	SAFE_RELEASE(m_pMp2DemuxVideoMap);
	SAFE_RELEASE(m_pMp2DemuxRefClock);
	SAFE_RELEASE(m_pMp2DemuxInterface);
	SAFE_RELEASE(m_pMp2DemuxFilter);

	SAFE_RELEASE(m_pSrcFilter);
	m_pBonSrcFilterClass=NULL;

	SAFE_RELEASE(m_pMediaControl);

#ifdef DEBUG
	if(m_dwRegister!=0){
		RemoveFromRot(m_dwRegister);
		m_dwRegister = 0;
	}
#endif

	TRACE(TEXT("FilterGraph RefCount = %d\n"),CDirectShowUtil::GetRefCount(m_pFilterGraph));
	SAFE_RELEASE(m_pFilterGraph);

	m_bInit=false;
}

const bool CMediaViewer::Play(void)
{
	if(!m_pMediaControl)return false;

	// tB^OtĐ
	m_pMediaControl->Pause();

	return (m_pMediaControl->Run() == S_OK)? true : false;
}

const bool CMediaViewer::Stop(void)
{
	if(!m_pMediaControl)return false;

	// tB^Ot~
	return (m_pMediaControl->Stop() == S_OK)? true : false;
}

const bool CMediaViewer::Pause()
{
	if (!m_pMediaControl)
		return false;
	return SUCCEEDED(m_pMediaControl->Pause());
}

const bool CMediaViewer::SetVideoPID(const WORD wPID)
{
	// fo̓sPID}bsO
	if(!m_pMp2DemuxVideoMap)return false;

	if(wPID == m_wVideoEsPID)return true;

	DWORD dwTempPID;

	// ݂PIDA}bv
	if(m_wVideoEsPID != 0xFFFFU){
		dwTempPID = (DWORD)m_wVideoEsPID;
		if(m_pMp2DemuxVideoMap->UnmapPID(1UL, &dwTempPID) != S_OK)return false;
		}

	// VKPID}bv
	if(wPID != 0xFFFFU){
		dwTempPID = wPID;
		if(m_pMp2DemuxVideoMap->MapPID(1UL, &dwTempPID, MEDIA_ELEMENTARY_STREAM) != S_OK)return false;
		}

	m_wVideoEsPID = wPID;

	return true;
}

const bool CMediaViewer::SetAudioPID(const WORD wPID)
{
	// o̓sPID}bsO
	if(!m_pMp2DemuxAudioMap)return false;

	if(wPID == m_wAudioEsPID)return true;

	DWORD dwTempPID;

	// ݂PIDA}bv
	if(m_wAudioEsPID != 0xFFFFU){
		dwTempPID = (DWORD)m_wAudioEsPID;
		if(m_pMp2DemuxAudioMap->UnmapPID(1UL, &dwTempPID) != S_OK)return false;
		}

	// VKPID}bv
	if(wPID != 0xFFFFU){
		dwTempPID = wPID;
		if(m_pMp2DemuxAudioMap->MapPID(1UL, &dwTempPID, MEDIA_ELEMENTARY_STREAM) != S_OK)return false;
		}

	m_wAudioEsPID = wPID;

	return true;
}

void CMediaViewer::OnMpeg2VideoInfo(const Mpeg2VideoInfo *pVideoInfo,const LPVOID pParam)
{
	// rfI̍XV
	CMediaViewer *pThis=static_cast<CMediaViewer*>(pParam);
	if(memcmp(&pThis->m_VideoInfo,pVideoInfo,sizeof(Mpeg2VideoInfo))){
		// rfI̍XV
		pThis->m_VideoInfo = *pVideoInfo;
		pThis->ResizeVideoWindow();
		}
}

const bool CMediaViewer::ResizeVideoWindow()
{
	// EBhETCYύX
	if((m_pVideoWindow && m_pBasicVideo)
#ifdef VMR9_SUPPORTED
			|| m_pVideoRenderer
#endif
			){
		WORD wX,wY;
		int AspectX,AspectY;
		long vx , vy;

		wX = m_wVideoWindowX;
		wY = m_wVideoWindowY;
		if((m_ForceAspectX>0 && m_ForceAspectY>0)
			|| (m_VideoInfo.byAspectRatioX>0 && m_VideoInfo.byAspectRatioY>0)){
			double aspect_rate;
			double window_rate = (double)wX / (double)wY;

			if (m_ForceAspectX>0 && m_ForceAspectY>0) {
				AspectX = m_ForceAspectX;
				AspectY = m_ForceAspectY;
			} else {
				AspectX = m_VideoInfo.byAspectRatioX;
				AspectY = m_VideoInfo.byAspectRatioY;
			}
			aspect_rate = (double)AspectX / (double)AspectY;
			if(aspect_rate>window_rate){
				vx = wX;
				vy = vx * AspectY  / AspectX;
			} else {
				vy = wY;
				vx = vy * AspectX / AspectY;
			}
		} else {
			vx = wX;
			vy = wY;
			AspectX=vx;
			AspectY=vy;
		}
		HRESULT hr;
		long SrcX,SrcY,SrcWidth,SrcHeight;

		CalcSourcePosition(&SrcX,&SrcY,&SrcWidth,&SrcHeight);
#ifdef VMR9_SUPPORTED
		if (m_pVideoRenderer) {

			IVMRWindowlessControl9 *pWindowlessControl;
			RECT rcSrc,rcDst;

			if (SUCCEEDED(m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
				LONG Width,Height;
				WORD VideoWidth,VideoHeight;

				if (SUCCEEDED(pWindowlessControl->GetNativeVideoSize(
												&Width,&Height,NULL,NULL))
						&& GetOriginalVideoSize(&VideoWidth,&VideoHeight)
						&& VideoWidth>0 && VideoHeight>0) {
					rcSrc.left=SrcX*Width/VideoWidth;
					rcSrc.top=SrcY*Height/VideoHeight;
					rcSrc.right=(SrcX+SrcWidth)*Width/VideoWidth;
					rcSrc.bottom=(SrcY+SrcHeight)*Height/VideoHeight;
				} else {
					rcSrc.left=SrcX;
					rcSrc.top=SrcY;
					rcSrc.right=SrcX+SrcWidth;
					rcSrc.bottom=SrcY+SrcHeight;
				}
				rcDst.left=(wX-vx)/2;
				rcDst.top=(wY-vy)/2,
				rcDst.right=rcDst.left+vx;
				rcDst.bottom=rcDst.top+vy;
				pWindowlessControl->SetVideoPosition(&rcSrc,&rcDst);
				pWindowlessControl->Release();
				InvalidateRect(m_hOwnerWnd,NULL,TRUE);
			}
		} else
#endif
		{
			hr = m_pBasicVideo->SetSourcePosition(SrcX,SrcY,SrcWidth,SrcHeight);
			hr = m_pBasicVideo->SetDestinationPosition((wX-vx)/2,(wY-vy)/2,vx,vy);
			//hr = m_pVideoWindow->HideCursor(m_bFullscreen?OATRUE:OAFALSE);
			hr = m_pVideoWindow->SetWindowPosition(0,0,wX,wY);
		}
		return true;
	}
	return false;
}

const bool CMediaViewer::SetViewSize(const int x,const int y)
{
	// EBhETCYݒ肷
	if(x>0 && y>0){
		m_wVideoWindowX = x;
		m_wVideoWindowY = y;
		return ResizeVideoWindow();
		}
	return false;
}


const bool CMediaViewer::SetVolume(const float fVolume)
{
	// I[fBI{[dBŐݒ肷( -100.0() < fVolume < 0(ő) )
	if(m_pBasicAudio){
		long lVolume = (long)(fVolume * 100);
		if(lVolume>=-10000&&lVolume<=0){
			TRACE(_T("Volume Control = %d\n"),lVolume);
			HRESULT hr = m_pBasicAudio->put_Volume(lVolume);
			if(SUCCEEDED(hr)){
				return true;
				}
			}
		}
	return false;
}

const bool CMediaViewer::GetVideoSize(WORD *pwWidth,WORD *pwHeight) const
{
	// rfĨTCY擾
	if(m_pMpeg2SeqClass){
		return m_pMpeg2SeqClass->GetVideoSize(pwWidth,pwHeight);
		}
	return false;
}

const bool CMediaViewer::GetVideoAspectRatio(BYTE *pbyAspectRatioX,BYTE *pbyAspectRatioY) const
{
	// rfĨAXyNg擾
	if(m_pMpeg2SeqClass){
		return m_pMpeg2SeqClass->GetAspectRatio(pbyAspectRatioX,pbyAspectRatioY);
		}
	return false;
}

const BYTE CMediaViewer::GetAudioChannelNum()
{
	// I[fBI̓̓`l擾
	if(m_pAacDecClass){
		return m_pAacDecClass->GetCurrentChannelNum();
		}
	return 0;
}

const bool CMediaViewer::SetStereoMode(const int iMode)
{
	// XeIo̓`l̐ݒ
	if(m_pPcmSelClass){
		m_pPcmSelClass->SetStereoMode(iMode);
		return true;
		}
	return false;
}

const bool CMediaViewer::GetVideoDecorderName(LPWSTR lpName,int iBufLen)
{
	// IĂrfIfR[_[̎擾
	if(m_pFilterGraph){
		if(m_strMpeg2DecorderName.GetLength()+1<iBufLen){
			lstrcpyW(lpName,m_strMpeg2DecorderName.GetBuffer());
			return true;
			}
		}
	return false;
}

const bool CMediaViewer::DisplayVideoDecorderProperty(HWND hWndParent)
{
	// rfIfR[_[̃vpeBJ
	bool bRet = false;
	FILTER_INFO FilterInfo;
	ISpecifyPropertyPages *pProp = NULL;
	IUnknown *pFilterUnk=NULL;;
	if(m_pMpeg2DecFilter){		
		HRESULT hr = m_pMpeg2DecFilter->QueryInterface(IID_ISpecifyPropertyPages, (void **)&pProp);
		if(SUCCEEDED(hr)){			
			CAUUID caGUID;
			pProp->GetPages(&caGUID);
			SAFE_RELEASE(pProp);
			hr = m_pMpeg2DecFilter->QueryFilterInfo(&FilterInfo);
			if(SUCCEEDED(hr)){				
				hr = m_pMpeg2DecFilter->QueryInterface(IID_IUnknown, (void **)&pFilterUnk);
				if(SUCCEEDED(hr)){
					OleCreatePropertyFrame(
						hWndParent,             // eEBhEB
						0, 0,                   // \ς݁B
						FilterInfo.achName,     // _CAO {bNX̃LvVB
						1,                      // IuWFNg (tB^̂)B
						&pFilterUnk,            // IuWFNg |C^̔zB
						caGUID.cElems,          // vpeB y[WB
						caGUID.pElems,          // vpeB y[W CLSID ̔zB
						0,                      // P[ʎqB
						0, NULL                 // \ς݁B
					);
					SAFE_RELEASE(pFilterUnk);
					bRet = true;
					}
				SAFE_RELEASE(FilterInfo.pGraph);
				}
			}
		}
	return bRet;
}

const bool CMediaViewer::DisplayVideoRanderProperty(HWND hWndParent)
{
	// IȂ̂ŁA
	return false;
}

HRESULT CMediaViewer::AddToRot(IUnknown *pUnkGraph, DWORD *pdwRegister) const
{
	// fobOp
	IMoniker * pMoniker;
	IRunningObjectTable *pROT;
	if(FAILED(::GetRunningObjectTable(0, &pROT)))return E_FAIL;
	
	WCHAR wsz[256];
	wsprintfW(wsz, L"FilterGraph %08p pid %08x", (DWORD_PTR)pUnkGraph, ::GetCurrentProcessId());

	HRESULT hr = ::CreateItemMoniker(L"!", wsz, &pMoniker);

	if(SUCCEEDED(hr)){
		hr = pROT->Register(0, pUnkGraph, pMoniker, pdwRegister);
		pMoniker->Release();
		}
	
	pROT->Release();
	
	return hr;
}

void CMediaViewer::RemoveFromRot(const DWORD dwRegister) const
{
	// fobOp
	IRunningObjectTable *pROT;

	if(SUCCEEDED(::GetRunningObjectTable(0, &pROT))){
		pROT->Revoke(dwRegister);
		pROT->Release();
		}
}




const bool CMediaViewer::ForceAspectRatio(int AspectX,int AspectY)
{
	m_ForceAspectX=AspectX;
	m_ForceAspectY=AspectY;
	return true;
}


const bool CMediaViewer::GetEffectiveAspectRatio(BYTE *pAspectX,BYTE *pAspectY)
{
	if (m_ForceAspectX!=0 && m_ForceAspectY!=0) {
		if (pAspectX)
			*pAspectX=m_ForceAspectX;
		if (pAspectY)
			*pAspectY=m_ForceAspectY;
		return true;
	}
	return GetVideoAspectRatio(pAspectX,pAspectY);
}


const bool CMediaViewer::SetPanAndScan(BYTE bFlags)
{
	m_PanAndScan=bFlags;
	return true;
}


const bool CMediaViewer::GetOriginalVideoSize(WORD *pWidth,WORD *pHeight) const
{
	if (m_pMpeg2SeqClass==NULL)
		return false;
	return m_pMpeg2SeqClass->GetOriginalVideoSize(pWidth,pHeight);
}


const bool CMediaViewer::GetDestRect(RECT *pRect)
{
#ifdef VMR9_SUPPORTED
	if (m_pVideoRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;
		HRESULT hr;

		if (FAILED(m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl))))
			return false;
		hr=pWindowlessControl->GetVideoPosition(NULL,pRect);
		pWindowlessControl->Release();
		return SUCCEEDED(hr);
	}
#endif
	{
		long Width,Height;

		if (m_pBasicVideo==NULL
				|| FAILED(m_pBasicVideo->GetDestinationPosition(
									&pRect->left,&pRect->top,&Width,&Height)))
			return false;
		pRect->right=pRect->left+Width;
		pRect->bottom=pRect->top+Height;
	}
	return true;
}


const bool CMediaViewer::GetDestSize(WORD *pWidth,WORD *pHeight)
{
	RECT rc;

	if (!GetDestRect(&rc))
		return false;
	if (pWidth)
		*pWidth=(WORD)(rc.right-rc.left);
	if (pHeight)
		*pHeight=(WORD)(rc.bottom-rc.top);
	return true;
}


const bool CMediaViewer::GetCroppedVideoSize(WORD *pWidth,WORD *pHeight)
{
	WORD Width,Height;

	Width=m_VideoInfo.wWidth;
	Height=m_VideoInfo.wHeight;
	if (m_PanAndScan&PANANDSCAN_HORZ)
		Width=m_VideoInfo.OrigWidth*12/16;
	if (m_PanAndScan&PANANDSCAN_VERT)
		Height=m_VideoInfo.OrigHeight*9/12;
	if (pWidth)
		*pWidth=Width;
	if (pHeight)
		*pHeight=Height;
	return true;
}


const bool CMediaViewer::GetSourceRect(RECT *pRect)
{
/*
#ifdef VMR9_SUPPORTED
	if (m_pVideoRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;
		HRESULT hr;

		if (FAILED(m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl))))
			return false;
		hr=pWindowlessControl->GetVideoPosition(pRect,NULL);
		pWindowlessControl->Release();
		return SUCCEEDED(hr);
	}
#endif
	{
		long Width,Height;

		if (m_pBasicVideo==NULL
				|| FAILED(m_pBasicVideo->GetSourcePosition(
									&pRect->left,&pRect->top,&Width,&Height)))
			return false;
		pRect->right=pRect->left+Width;
		pRect->bottom=pRect->top+Height;
	}
	return true;
*/
	if (!CalcSourcePosition(&pRect->left,&pRect->top,
												&pRect->right,&pRect->bottom))
		return false;
	pRect->right+=pRect->left;
	pRect->bottom+=pRect->top;
	return true;
}


const void CMediaViewer::HideCursor(bool bHide)
{
	if (m_pVideoWindow)
		m_pVideoWindow->HideCursor(bHide?OATRUE:OAFALSE);
}


const bool CMediaViewer::CalcSourcePosition(long *pLeft,long *pTop,
											long *pWidth,long *pHeight) const
{
	long SrcX,SrcY,SrcWidth,SrcHeight;

	if (m_PanAndScan&PANANDSCAN_HORZ) {
		SrcWidth=m_VideoInfo.OrigWidth*12/16;
		SrcX=(m_VideoInfo.OrigWidth-SrcWidth)/2;
	} else {
		SrcWidth=m_VideoInfo.wWidth;
		SrcX=m_VideoInfo.wPosX;
	}
	if (m_PanAndScan&PANANDSCAN_VERT) {
		SrcHeight=m_VideoInfo.OrigHeight*9/12;
		SrcY=(m_VideoInfo.OrigHeight-SrcHeight)/2;
	} else {
		SrcHeight=m_VideoInfo.wHeight;
		SrcY=m_VideoInfo.wPosY;
	}
	if (pLeft)
		*pLeft=SrcX;
	if (pTop)
		*pTop=SrcY;
	if (pWidth)
		*pWidth=SrcWidth;
	if (pHeight)
		*pHeight=SrcHeight;
	return true;
}


const bool CMediaViewer::GetCurrentImage(BYTE **ppDib)
{
	bool fOK=false;

#ifdef VMR9_SUPPORTED
	if (m_pVideoRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;

		if (SUCCEEDED(m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			fOK=SUCCEEDED(pWindowlessControl->GetCurrentImage(ppDib));
			pWindowlessControl->Release();
		}
	} else
#endif
	if (m_pBasicVideo) {
		long Size;

		if (SUCCEEDED(m_pBasicVideo->GetCurrentImage(&Size,NULL)) && Size>0) {
			*ppDib=(BYTE*)CoTaskMemAlloc(Size);
			if (*ppDib!=NULL) {
				if (SUCCEEDED(m_pBasicVideo->GetCurrentImage(&Size,
											reinterpret_cast<long*>(*ppDib)))) {
					fOK=true;
				} else {
					CoTaskMemFree(*ppDib);
					*ppDib=NULL;
				}
			}
		}
	}
	return fOK;
}


bool CMediaViewer::SetGrabber(bool bGrabber)
{
	m_bGrabber=bGrabber;
	return true;
}


void *CMediaViewer::DoCapture(DWORD WaitTime)
{
#if 1
	void *pDib;

	if (m_pGrabber==NULL)
		return NULL;
	if (!m_pGrabber->SetCapture(true))
		return NULL;
	if (!m_pGrabber->WaitCapture(WaitTime)) {
		m_pGrabber->SetCapture(false);
		return NULL;
	}
	pDib=m_pGrabber->GetCaptureBitmap();
	m_pGrabber->SetCapture(false);
	return pDib;
#else
	return NULL;
#endif
}


bool CMediaViewer::SetAudioNormalize(bool bNormalize,float Level)
{
	if (m_pAacDecClass==NULL)
		return false;
	return m_pAacDecClass->SetNormalize(bNormalize,Level);
}


#ifdef VMR9_SUPPORTED


const bool CMediaViewer::SetVMR9(bool bVMR9)
{
	m_bVMR9=bVMR9;
	return true;
}


const bool CMediaViewer::RepaintVideo(HWND hwnd,HDC hdc)
{
	if (m_pVideoRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;

		if (SUCCEEDED(m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			pWindowlessControl->RepaintVideo(hwnd,hdc);
			pWindowlessControl->Release();
		}
	}
	return true;
}


const bool CMediaViewer::DisplayModeChanged()
{
	if (m_pVideoRenderer) {
		IVMRWindowlessControl9 *pWindowlessControl;

		if (SUCCEEDED(m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl)))) {
			pWindowlessControl->DisplayModeChanged();
			pWindowlessControl->Release();
		}
	}
	return true;
}


const bool CMediaViewer::DrawText(LPCTSTR pszText,HFONT hfont,COLORREF crColor,
												int Opacity,RECT *pDestRect)
{
	IVMRWindowlessControl9 *pWindowlessControl;
	WORD VideoWidth,VideoHeight;
	LONG Width,Height;
	RECT rc;

	if (m_pVideoRenderer==NULL)
		return false;
	if (m_pImageMixer==NULL)
		m_pImageMixer=new CImageMixer(m_pVideoRenderer);
	if (FAILED(m_pVideoRenderer->QueryInterface(IID_IVMRWindowlessControl9,
							reinterpret_cast<LPVOID*>(&pWindowlessControl))))
		return false;
	pWindowlessControl->GetNativeVideoSize(&Width,&Height,NULL,NULL);
	pWindowlessControl->Release();
	GetVideoSize(&VideoWidth,&VideoHeight);
	rc.left=pDestRect->left*Width/VideoWidth;
	rc.top=pDestRect->top*Height/VideoHeight;
	rc.right=pDestRect->right*Width/VideoWidth;
	rc.bottom=pDestRect->bottom*Height/VideoHeight;
	return m_pImageMixer->SetText(pszText,hfont,crColor,Opacity,&rc);
}


const bool CMediaViewer::ClearOSD()
{
	if (m_pVideoRenderer==NULL)
		return false;
	if (m_pImageMixer!=NULL)
		m_pImageMixer->Clear();
	return true;
}


#endif	// VMR9_SUPPORTED


bool CMediaViewer::SetTracer(CTracer *pTracer)
{
	m_pTracer=pTracer;
	return true;
}


void CMediaViewer::Trace(LPCTSTR pszOutput, ...)
{
	va_list Args;

	va_start(Args,pszOutput);
	if (m_pTracer!=NULL)
		m_pTracer->TraceV(pszOutput,Args);
	va_end(Args);
}
