#ifndef COLOR_SCHEME_H
#define COLOR_SCHEME_H


#include "ColorPalette.h"


class CColorScheme {
public:
	enum {
		COLOR_STATUSBACK,
		COLOR_STATUSTEXT,
		COLOR_STATUSHIGHLIGHTBACK,
		COLOR_STATUSHIGHLIGHTTEXT,
		COLOR_PANELBACK,
		COLOR_PANELTEXT,
		COLOR_PANELTABBACK,
		COLOR_PANELTABTEXT,
		COLOR_PANELTABBORDER,
		COLOR_PANELCURTABBACK,
		COLOR_PANELCURTABTEXT,
		COLOR_PANELCURTABBORDER,
		COLOR_PANELTABMARGIN,
		COLOR_PANELTITLEBACK,
		COLOR_PANELTITLETEXT,
		COLOR_PROGRAMINFOBACK,
		COLOR_PROGRAMINFOTEXT,
		COLOR_PROGRAMLISTBACK,
		COLOR_PROGRAMLISTTEXT,
		COLOR_PROGRAMLISTTITLEBACK,
		COLOR_PROGRAMLISTTITLETEXT,
		COLOR_CONTROLPANELHIGHLIGHTBACK,
		COLOR_CONTROLPANELHIGHLIGHTTEXT
	};
	enum { NUM_COLORS=23 };
	CColorScheme();
	~CColorScheme();
	COLORREF GetColor(int Type) const;
	bool SetColor(int Type,COLORREF Color);
	LPCTSTR GetName() const { return m_pszName; }
	bool SetName(LPCTSTR pszName);
	LPCTSTR GetFileName() const { return m_pszFileName; }
	bool Load(LPCTSTR pszFileName);
	bool Save(LPCTSTR pszFileName) const;
	bool SetFileName(LPCTSTR pszFileName);
	void SetDefault();
	static LPCTSTR GetColorName(int Type);
	static COLORREF GetDefaultColor(int Type);
private:
	COLORREF m_ColorList[NUM_COLORS];
	LPTSTR m_pszName;
	LPTSTR m_pszFileName;
	struct ColorInfo {
		COLORREF DefaultColor;
		LPCTSTR pszText;
		LPCTSTR pszName;
	};
	static const ColorInfo m_ColorInfoList[NUM_COLORS];
};

class CColorSchemeList {
	int m_NumColorSchemes;
	CColorScheme **m_ppList;
public:
	CColorSchemeList();
	~CColorSchemeList();
	int NumColorSchemes() const { return m_NumColorSchemes; }
	bool Add(CColorScheme *pColorScheme);
	bool Load(LPCTSTR pszDirectory);
	void Clear();
	CColorScheme *GetColorScheme(int Index);
};

class CColorSchemeOptions {
public:
	typedef bool (*ApplyFunc)(const CColorScheme *pColorScheme);
private:
	CColorScheme *m_pColorScheme;
	CColorSchemeList m_PresetList;
	bool m_fPreview;
	ApplyFunc m_pApplyFunc;
	CColorPalette m_ColorPalette;
	bool Apply(const CColorScheme *pColorScheme) const;
	static void GetListColor(HWND hDlg,CColorScheme *pColorScheme);
	static void SetListColor(HWND hDlg,const CColorScheme *pColorScheme);
	static const LPCTSTR m_pszExtension;
	static CColorSchemeOptions *GetThis(HWND hwnd);
public:
	CColorSchemeOptions();
	~CColorSchemeOptions();
	bool Load(LPCTSTR pszFileName);
	bool Save(LPCTSTR pszFileName) const;
	bool SetApplyCallback(ApplyFunc pCallback);
	bool ApplyColorScheme() const;
	static bool GetThemesDirectory(LPTSTR pszDirectory,int MaxLength,bool fCreate=false);
	static BOOL CALLBACK DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
};


#endif
